@@ -16,7 +16,8 @@
 module Hledger.Web.Foundation where
 
 import Control.Applicative ((<|>))
-import Control.Monad (join, when)
+import Control.Monad (join, when, unless)
+-- import Control.Monad.Except (runExceptT)  -- now re-exported by Hledger
 import qualified Data.ByteString.Char8 as BC
 import Data.Traversable (for)
 import Data.IORef (IORef, readIORef, writeIORef)
@@ -60,6 +61,8 @@ data App = App
       --
     , appOpts    :: WebOpts
     , appJournal :: IORef Journal
+        -- ^ the current journal, filtered by the initial command line query
+        --   but ignoring any depth limit.
     }
 
 
@@ -83,16 +86,17 @@ data App = App
 -- usually require access to the AppRoute datatype. Therefore, we
 -- split these actions into two functions and place them in separate files.
 mkYesodData "App" $(parseRoutesFile "config/routes")
+-- ^ defines things like:
+-- type Handler = HandlerFor App   -- HandlerT App IO, https://www.yesodweb.com/book/routing-and-handlers#routing-and-handlers_handler_monad
+-- type Widget = WidgetFor App ()  -- WidgetT App IO (), https://www.yesodweb.com/book/widgets
 
--- | A convenience alias.
 type AppRoute = Route App
-
-type Form x = Html -> MForm (HandlerFor App) (FormResult x, Widget)
+type Form a = Html -> MForm Handler (FormResult a, Widget)
 
 -- Please see the documentation for the Yesod typeclass. There are a number
 -- of settings which can be configured by overriding methods here.
 instance Yesod App where
-  approot = ApprootMaster $ appRoot . settings
+  approot = guessApprootOr (ApprootMaster $ appRoot . settings)
 
   makeSessionBackend _ = do
     hledgerdata <- getXdgDirectory XdgCache "hledger"
@@ -110,20 +114,25 @@ instance Yesod App where
 
     master <- getYesod
     here <- fromMaybe RootR <$> getCurrentRoute
-    VD {caps, j, m, opts, q, qopts} <- getViewData
+    VD{opts, j, qparam, q, qopts, perms} <- getViewData
     msg <- getMessage
     showSidebar <- shouldShowSidebar
-    hideEmptyAccts <- (== Just "1") . lookup "hideemptyaccts" . reqCookies <$> getRequest
 
     let rspec = reportspec_ (cliopts_ opts)
-        ropts = rsOpts rspec
-        ropts' = (rsOpts rspec)
+        ropts = _rsReportOpts rspec
+        ropts' = (_rsReportOpts rspec)
           {accountlistmode_ = ALTree  -- force tree mode for sidebar
-          ,empty_           = not (empty_ ropts)  -- show zero items by default
+          ,empty_           = True    -- show zero items by default
           }
-        rspec' = rspec{rsQuery=m,rsOpts=ropts'}
-        accounts =
-          balanceReportAsHtml (JournalR, RegisterR) here hideEmptyAccts j q qopts $
+        rspec' = rspec{_rsQuery=q,_rsReportOpts=ropts'}
+
+    hideEmptyAccts <- if empty_ ropts
+                         then return True
+                         else (== Just "1") . lookup "hideemptyaccts" . reqCookies <$> getRequest
+
+    let accounts =
+          balanceReportAsHtml (JournalR, RegisterR) here hideEmptyAccts j qparam qopts $
+          styleAmounts (journalCommodityStylesWith HardRounding j) $
           balanceReport rspec' j
 
         topShowmd = if showSidebar then "col-md-4" else "col-any-0" :: Text
@@ -187,10 +196,10 @@ data ViewData = VD
   { opts  :: WebOpts    -- ^ the command-line options at startup
   , today :: Day        -- ^ today's date (for queries containing relative dates)
   , j     :: Journal    -- ^ the up-to-date parsed unfiltered journal
-  , q     :: Text       -- ^ the current q parameter, the main query expression
-  , m     :: Query      -- ^ a query parsed from the q parameter
+  , qparam :: Text       -- ^ the current "q" request parameter
+  , q     :: Query      -- ^ a query parsed from the q parameter
   , qopts :: [QueryOpt] -- ^ query options parsed from the q parameter
-  , caps  :: [Capability] -- ^ capabilities enabled for this request
+  , perms :: [Permission]  -- ^ permissions enabled for this request (by --allow and/or X-Sandstorm-Permissions)
   } deriving (Show)
 
 instance Show Text.Blaze.Markup where show _ = "<blaze markup>"
@@ -198,36 +207,46 @@ instance Show Text.Blaze.Markup where show _ = "<blaze markup>"
 -- | Gather data used by handlers and templates in the current request.
 getViewData :: Handler ViewData
 getViewData = do
-  App{appOpts=opts@WebOpts{cliopts_=copts@CliOpts{reportspec_=rspec@ReportSpec{rsOpts}}}, appJournal} <- getYesod
-  today <- liftIO getCurrentDay
+  App{
+    appOpts=opts@WebOpts{ cliopts_=copts@CliOpts{ reportspec_=rspec@ReportSpec{_rsReportOpts, _rsQuery} } },
+    appJournal
+  } <- getYesod
+  let today = _rsDay rspec
 
   -- try to read the latest journal content, keeping the old content
   -- if there's an error
   (j, mjerr) <- getCurrentJournal
                 appJournal
-                copts{reportspec_=rspec{rsOpts=rsOpts{no_elide_=True}}}
+                copts{reportspec_=rspec{_rsReportOpts=_rsReportOpts{no_elide_=True}}}
                 today
 
-  -- try to parse the query param, assuming no query if there's an error
-  q <- fromMaybe "" <$> lookupGetParam "q"
-  (m, qopts, mqerr) <- do
-    case parseQuery today q of
-      Right (m, qopts) -> return (m, qopts, Nothing)
+  -- Get the query specified by the q request parameter, or no query if this fails.
+  qparam <- fromMaybe "" <$> lookupGetParam "q"
+  (q1, qopts, mqerr) <- do
+    case parseQuery today qparam of
+      Right (q0, qopts) -> return (q0, qopts, Nothing)
       Left err         -> return (Any, [], Just err)
+  -- To this, add any depth limit from the initial startup query, preserving that.
+  let
+    initialdepthq = filterQuery queryIsDepth _rsQuery
+    q = simplifyQuery $ And [q1, initialdepthq]
 
   -- if either of the above gave an error, display it
   maybe (pure ()) (setMessage . toHtml) $ mjerr <|> mqerr
 
-  -- do some permissions checking
-  caps <- case capabilitiesHeader_ opts of
-    Nothing -> return (capabilities_ opts)
-    Just h -> do
+  -- find out which permissions are enabled
+  perms <- case allow_ opts of
+    -- if started with --allow=sandstorm, take permissions from X-Sandstorm-Permissions header
+    SandstormAccess -> do
+      let h = "X-Sandstorm-Permissions"
       hs <- fmap (BC.split ',' . snd) . filter ((== h) . fst) . requestHeaders <$> waiRequest
-      fmap join . for (join hs) $ \x -> case capabilityFromBS x of
-        Left e -> [] <$ addMessage "" ("Unknown permission: " <> toHtml (BC.unpack e))
-        Right c -> pure [c]
+      fmap join . for (join hs) $ \x -> case parsePermission x of
+        Left  e -> [] <$ addMessage "" ("Unknown permission: " <> toHtml e)
+        Right p -> pure [p]
+    -- otherwise take them from the access level specified by --allow's access level
+    cliaccess -> pure $ accessLevelToPermissions cliaccess
 
-  return VD{opts, today, j, q, m, qopts, caps}
+  return VD{opts, today, j, qparam, q, qopts, perms}
 
 checkServerSideUiEnabled :: Handler ()
 checkServerSideUiEnabled = do
@@ -245,26 +264,31 @@ shouldShowSidebar = do
   msidebarcookie <- lookup "showsidebar" . reqCookies <$> getRequest
   return $
     let disablevalues = ["","0"]
-    in maybe
-         (not $ msidebarcookie `elem` map Just disablevalues)
-         (not . (`elem` disablevalues))
-         msidebarparam
+    in maybe True (`notElem` disablevalues) $ msidebarparam <|> msidebarcookie
 
 -- | Update our copy of the journal if the file changed. If there is an
 -- error while reloading, keep the old one and return the error, and set a
 -- ui message.
 getCurrentJournal :: IORef Journal -> CliOpts -> Day -> Handler (Journal, Maybe String)
 getCurrentJournal jref opts d = do
+  -- re-apply any initial filter specified at startup
+  let depthlessinitialq = filterQuery (not . queryIsDepth) $ _rsQuery $ reportspec_ opts
   -- XXX put this inside atomicModifyIORef' for thread safety
   j <- liftIO (readIORef jref)
-  (ej, changed) <- liftIO $ journalReloadIfChanged opts d j
-  -- re-apply any initial filter specified at startup
-  let initq = rsQuery $ reportspec_ opts
-  case (changed, filterJournalTransactions initq <$> ej) of
-    (False, _) -> return (j, Nothing)
-    (True, Right j') -> do
-      liftIO $ writeIORef jref j'
-      return (j',Nothing)
-    (True, Left e) -> do
+  ej <- liftIO . runExceptT $ journalReloadIfChanged opts d j
+  case ej of
+    Left e -> do
       setMessage "error while reading journal"
       return (j, Just e)
+    Right (j', True) -> do
+      liftIO . writeIORef jref $ filterJournalTransactions depthlessinitialq j'
+      return (j',Nothing)
+    Right (_, False) -> return (j, Nothing)
+
+-- | In a request handler, check for the given permission
+-- and fail with a message if it's not present.
+require :: Permission -> Handler ()
+require p = do
+  VD{perms} <- getViewData
+  unless (p `elem` perms) $ permissionDenied $
+    "Missing the '" <> T.pack (showPermission p) <> "' permission"
