@@ -1,10 +1,11 @@
 -- | /register handlers.
 
-{-# LANGUAGE NamedFieldPuns #-}
-{-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE QuasiQuotes #-}
+{-# LANGUAGE NamedFieldPuns      #-}
+{-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE QuasiQuotes         #-}
 {-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TupleSections       #-}
 
 module Hledger.Web.Handler.RegisterR where
 
@@ -25,28 +26,30 @@ import Hledger.Web.Widget.Common
 getRegisterR :: Handler Html
 getRegisterR = do
   checkServerSideUiEnabled
-  VD{caps, j, m, opts, q, qopts, today} <- getViewData
-  when (CapView `notElem` caps) (permissionDenied "Missing the 'view' capability")
+  VD{perms, j, q, opts, qparam, qopts, today} <- getViewData
+  require ViewPermission
 
   let (a,inclsubs) = fromMaybe ("all accounts",True) $ inAccount qopts
       s1 = if inclsubs then "" else " (excluding subaccounts)"
-      s2 = if m /= Any then ", filtered" else ""
+      s2 = if q /= Any then ", filtered" else ""
       header = a <> s1 <> s2
 
   let rspec = reportspec_ (cliopts_ opts)
       acctQuery = fromMaybe Any (inAccountQuery qopts)
-      acctlink acc = (RegisterR, [("q", replaceInacct q $ accountQuery acc)])
+      acctlink acc = (RegisterR, [("q", replaceInacct qparam $ accountQuery acc)])
       otherTransAccounts =
           map (\(acct,(name,comma)) -> (acct, (T.pack name, T.pack comma))) .
           undecorateLinks . elideRightDecorated 40 . decorateLinks .
-          addCommas . preferReal . otherTransactionAccounts m acctQuery
+          addCommas . preferReal . otherTransactionAccounts q acctQuery
       addCommas xs =
           zip xs $
           zip (map (T.unpack . accountSummarisedName . paccount) xs) $
           tail $ (", "<$xs) ++ [""]
-      items = accountTransactionsReport rspec j m acctQuery
+      items =
+        styleAmounts (journalCommodityStylesWith HardRounding j) $
+        accountTransactionsReport rspec{_rsQuery=q} j acctQuery
       balancelabel
-        | isJust (inAccount qopts), balancetype_ (rsOpts rspec) == HistoricalBalance = "Historical Total"
+        | isJust (inAccount qopts), balanceaccum_ (_rsReportOpts rspec) == Historical = "Historical Total"
         | isJust (inAccount qopts) = "Period Total"
         | otherwise                = "Total"
       transactionFrag = transactionFragment j
@@ -78,7 +81,7 @@ preferReal ps
 elideRightDecorated :: Int -> [(Maybe d, Char)] -> [(Maybe d, Char)]
 elideRightDecorated width s =
     if length s > width
-        then take (width - 2) s ++ map ((,) Nothing) ".."
+        then take (width - 2) s ++ map (Nothing,) ".."
         else s
 
 undecorateLinks :: [(Maybe acct, char)] -> [(acct, ([char], [char]))]
@@ -92,14 +95,12 @@ undecorateLinks xs0@(x:_) =
         _ -> error "link name not decorated with account"  -- PARTIAL:
 
 decorateLinks :: [(acct, ([char], [char]))] -> [(Maybe acct, char)]
-decorateLinks =
-    concatMap
-        (\(acct, (name, comma)) ->
-            map ((,) (Just acct)) name ++ map ((,) Nothing) comma)
+decorateLinks = concatMap $ \(acct, (name, comma)) ->
+    map (Just acct,) name ++ map (Nothing,) comma
 
 -- | Generate javascript/html for a register balance line chart based on
--- the provided "TransactionsReportItem"s.
-registerChartHtml :: Text -> String -> [(CommoditySymbol, [TransactionsReportItem])] -> HtmlUrl AppRoute
+-- the provided "AccountTransactionsReportItem"s.
+registerChartHtml :: Text -> String -> [(CommoditySymbol, [AccountTransactionsReportItem])] -> HtmlUrl AppRoute
 registerChartHtml q title percommoditytxnreports = $(hamletFile "templates/chart.hamlet")
  -- have to make sure plot is not called when our container (maincontent)
  -- is hidden, eg with add form toggled
